==> ubuntuone-client_1.0.2.orig.tar.gz: ubuntuone-client-1.0.2/tests/syncdaemon/test_config.py <==
#
# Author: Guillermo Gonzalez <guillermo.gonzalez@canonical.com>
#
# Copyright 2009 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""tests for the syncdaemon config module """
from __future__ import with_statement

import os
from ConfigParser import ConfigParser

from ubuntuone.syncdaemon import (
    config,
)
from contrib.testing.testcase import (
    BaseTwistedTestCase,
)


class TestConfigBasic(BaseTwistedTestCase):
    """Basic _Config object tests"""

    def setUp(self):
        BaseTwistedTestCase.setUp(self)
        self.test_root = self.mktemp()

    def tearDown(self):
        """cleanup everything, also the config module globals"""
        BaseTwistedTestCase.tearDown(self)
        config._user_config = None

    def assertThrottlingSection(self, expected, current, on, read, write):
        """Assert for equality two ConfigParser and against the on, read and
        write args
        """
        self.assertEquals(expected.getboolean(config.THROTTLING, 'on'), on)
        self.assertEquals(expected.getint(config.THROTTLING, 'read_limit'),
                          read)
        self.assertEquals(expected.getint(config.THROTTLING, 'write_limit'),
                          write)
        self.assertEquals(expected.getboolean(config.THROTTLING, 'on'),
                          current.get_throttling())
        self.assertEquals(expected.getint(config.THROTTLING, 'read_limit'),
                          current.get_throttling_read_limit())
        self.assertEquals(expected.getint(config.THROTTLING, 'write_limit'),
                          current.get_throttling_write_limit())

    def test_load_empty(self):
        """test loading the a non-existent config file"""
        conf_file = os.path.join(self.test_root, 'test_missing_config.conf')
        # create the config object with an empty config file
        conf = config._Config(conf_file)
        # lie a bit and load the defaults, as we can't access the data
        # dir to get the current config, and a test shouldn't depend on
        # external data, just fake all the defaults
        default_conf = os.path.join(self.test_root, 'defaults.conf')
        with open(default_conf, 'w') as fp:
            fp.write('[bandwidth_throttling]\n')
            fp.write('on.default = False\n')
            fp.write('read_limit.default = -1\n')
            fp.write('write_limit.default = -1\n')
        conf.default = config._Config._load_defaults([default_conf])
        self.assertEquals(False, conf.get_throttling())
        self.assertEquals(None, conf.get_throttling_read_limit())
        self.assertEquals(None, conf.get_throttling_write_limit())

    def test_load_basic(self):
        """test loading the config file containing only the throttling values"""
        conf_file = os.path.join(self.test_root, 'test_load_config.conf')
        # write some throttling values to the config file
        with open(conf_file, 'w') as fp:
            fp.write('[bandwidth_throttling]\n')
            fp.write('on = True\n')
            fp.write('read_limit = 1000\n')
            fp.write('write_limit = 200\n')
        conf = config._Config(conf_file)
        self.assertEquals(True, conf.get_throttling())
        self.assertEquals(1000, conf.get_throttling_read_limit())
        self.assertEquals(200, conf.get_throttling_write_limit())

    def test_load_extra_data(self):
        """test loading the a config file with other sections too"""
        conf_file = os.path.join(self.test_root, 'test_load_extra_config.conf')
        # write some throttling values to the config file
        with open(conf_file, 'w') as fp:
            fp.write('[__main__]\n')
            fp.write('log_level = INFO\n')
            fp.write('disable_ssl_verify = True\n')
            fp.write('\n')
            fp.write('[bandwidth_throttling]\n')
            fp.write('on = True\n')
            fp.write('read_limit = 1000\n')
            fp.write('write_limit = 200\n')
        conf = config._Config(conf_file)
        self.assertEquals(True, conf.get_throttling())
        self.assertEquals(1000, conf.get_throttling_read_limit())
        self.assertEquals(200, conf.get_throttling_write_limit())

    def test_write_new(self):
        """test writing the throttling section to a new config file"""
        conf_file = os.path.join(self.test_root, 'test_write_new_config.conf')
        self.assertFalse(os.path.exists(conf_file))
        conf = config._Config(conf_file)
        conf.set_throttling(True)
        conf.set_throttling_read_limit(1000)
        conf.set_throttling_write_limit(100)
        conf.save()
        # load the config in a barebone ConfigParser and check
        conf_1 = ConfigParser()
        conf_1.read(conf_file)
        self.assertThrottlingSection(conf_1, conf, True, 1000, 100)

    def test_write_existing(self):
        """test writing the throttling section to a existing config file"""
        conf_file = os.path.join(self.test_root,
                                 'test_write_existing_config.conf')
        # write some throttling values to the config file
        with open(conf_file, 'w') as fp:
            fp.write('[bandwidth_throttling]\n')
            fp.write('on = False\n')
            fp.write('read_limit = 1000\n')
            fp.write('write_limit = 100\n')
        self.assertTrue(os.path.exists(conf_file))
        conf = config._Config(conf_file)
        conf.set_throttling(True)
        conf.set_throttling_read_limit(2000)
        conf.set_throttling_write_limit(200)
        conf.save()
        # load the config in a barebone ConfigParser and check
        conf_1 = ConfigParser()
        conf_1.read(conf_file)
        self.assertThrottlingSection(conf_1, conf, True, 2000, 200)

    def test_write_extra(self):
        """test writing the throttling section back to the config file,
        including extra sections
        """
        conf_file = os.path.join(self.test_root, 'test_write_extra_config.conf')
        # write some throttling values to the config file
        with open(conf_file, 'w') as fp:
            fp.write('[__main__]\n')
            fp.write('log_level = INFO\n')
            fp.write('disable_ssl_verify = True\n')
            fp.write('\n')
            fp.write('[bandwidth_throttling]\n')
            fp.write('on = False\n')
            fp.write('read_limit = 2000\n')
            fp.write('write_limit = 200\n')
        self.assertTrue(os.path.exists(conf_file))
        conf = config._Config(conf_file)
        conf.set_throttling(True)
        conf.set_throttling_read_limit(3000)
        conf.set_throttling_write_limit(300)
        conf.save()
        # load the config in a barebone ConfigParser and check
        conf_1 = ConfigParser()
        conf_1.read(conf_file)
        self.assertThrottlingSection(conf_1, conf, True, 3000, 300)
        self.assertEquals(conf_1.get('__main__', 'log_level'),
                          conf.get('__main__', 'log_level'))
        self.assertEquals(conf_1.getboolean('__main__', 'disable_ssl_verify'),
                          conf.getboolean('__main__', 'disable_ssl_verify'))

    def test_write_existing_partial(self):
        """test writing a partially updated throttling section
        to a existing config file
        """
        conf_file = os.path.join(self.test_root,
                                 'test_write_existing_config.conf')
        # write some throttling values to the config file
        with open(conf_file, 'w') as fp:
            fp.write('[bandwidth_throttling]\n')
            fp.write('on = True\n')
            fp.write('read_limit = 1000\n')
            fp.write('write_limit = 100\n')
        self.assertTrue(os.path.exists(conf_file))
        conf = config._Config(conf_file)
        conf.set_throttling(False)
        conf.save()
        # load the config in a barebone ConfigParser and check
        conf_1 = ConfigParser()
        conf_1.read(conf_file)
        self.assertThrottlingSection(conf_1, conf, False, 1000, 100)


    def test_load_negative_limits(self):
        """test loading the config file with negative read/write limits"""
        conf_file = os.path.join(self.test_root, 'test_load_config.conf')
        # write some throttling values to the config file
        with open(conf_file, 'w') as fp:
            fp.write('[bandwidth_throttling]\n')
            fp.write('on = True\n')
            fp.write('read_limit = -1\n')
            fp.write('write_limit = -1\n')
        conf = config._Config(conf_file)
        self.assertEquals(True, conf.get_throttling())
        self.assertEquals(None, conf.get_throttling_read_limit())
        self.assertEquals(None, conf.get_throttling_write_limit())


    def test_load_partial_config(self):
        """test loading a partial config file and fallback to defaults"""
        conf_file = os.path.join(self.test_root, 'test_load_config.conf')
        # write some throttling values to the config file
        with open(conf_file, 'w') as fp:
            fp.write('[bandwidth_throttling]\n')
            fp.write('on = True\n')
            fp.write('read_limit = 1\n')
        conf = config._Config(conf_file)
        # lie a bit and load the defaults, as we can't access the data
        # dir to get the current config, and a test shouldn't depend on
        # external data, just fake all the defaults
        default_conf = os.path.join(self.test_root, 'defaults.conf')
        with open(default_conf, 'w') as fp:
            fp.write('[bandwidth_throttling]\n')
            fp.write('on.default = False\n')
            fp.write('read_limit.default = -1\n')
            fp.write('write_limit.default = -1\n')
        conf.default = config._Config._load_defaults([default_conf])
        self.assertEquals(True, conf.get_throttling())
        self.assertEquals(1, conf.get_throttling_read_limit())
        self.assertEquals(None, conf.get_throttling_write_limit())

==> ubuntuone-client_1.0.3.orig.tar.gz: ubuntuone-client-1.0.3/tests/syncdaemon/test_config.py <==
#
# Author: Guillermo Gonzalez <guillermo.gonzalez@canonical.com>
#
# Copyright 2009 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""tests for the syncdaemon config module """
from __future__ import with_statement

import contextlib
import logging
import os
from ConfigParser import ConfigParser
from xdg.BaseDirectory import (
    xdg_data_home,
    xdg_cache_home,
)

from ubuntuone.syncdaemon import (
    config,
)
from contrib.testing.testcase import (
    BaseTwistedTestCase,
    DBusTwistedTestCase,
)


@contextlib.contextmanager
def environ(env_var, new_value):
    """context manager to replace/add an environ value"""
    old_value = os.environ.get(env_var, None)
    os.environ[env_var] = new_value
    yield
    if old_value is None:
        os.environ.pop(env_var)
    else:
        os.environ[env_var] = old_value


class TestConfigBasic(BaseTwistedTestCase):
    """Basic _Config object tests"""

    def setUp(self):
        BaseTwistedTestCase.setUp(self)
        self.test_root = self.mktemp()
        self.old_get_config_files = config.get_config_files
        config.get_config_files = lambda: [os.path.join(os.environ['ROOT_DIR'],
                                                   'data', 'syncdaemon.conf')]

    def tearDown(self):
        """cleanup everything, also the config module globals"""
        config._user_config = None
        config.get_config_files = self.old_get_config_files
        BaseTwistedTestCase.tearDown(self)

    def assertThrottlingSection(self, expected, current, on, read, write):
        """Assert for equality two ConfigParser and against the on, read and
        write args
        """
        self.assertEquals(expected.getboolean(config.THROTTLING, 'on'), on)
        self.assertEquals(expected.getint(config.THROTTLING, 'read_limit'),
                          read)
        self.assertEquals(expected.getint(config.THROTTLING, 'write_limit'),
                          write)
        self.assertEquals(expected.getboolean(config.THROTTLING, 'on'),
                          current.get_throttling())
        self.assertEquals(expected.getint(config.THROTTLING, 'read_limit'),
                          current.get_throttling_read_limit())
        self.assertEquals(expected.getint(config.THROTTLING, 'write_limit'),
                          current.get_throttling_write_limit())

    def test_load_empty(self):
        """test loading the a non-existent config file"""
        conf_file = os.path.join(self.test_root, 'test_missing_config.conf')
        # create the config object with an empty config file
        conf = config._Config(conf_file)
        self.assertEquals(False, conf.get_throttling())
        self.assertEquals(None, conf.get_throttling_read_limit())
        self.assertEquals(None, conf.get_throttling_write_limit())

    def test_load_basic(self):
        """test loading the config file containing only the throttling values"""
        conf_file = os.path.join(self.test_root, 'test_load_config.conf')
        # write some throttling values to the config file
        with open(conf_file, 'w') as fp:
            fp.write('[bandwidth_throttling]\n')
            fp.write('on = True\n')
            fp.write('read_limit = 1000\n')
            fp.write('write_limit = 200\n')
        conf = config._Config(conf_file)
        self.assertEquals(True, conf.get_throttling())
        self.assertEquals(1000, conf.get_throttling_read_limit())
        self.assertEquals(200, conf.get_throttling_write_limit())

    def test_load_extra_data(self):
        """test loading the a config file with other sections too"""
        conf_file = os.path.join(self.test_root, 'test_load_extra_config.conf')
        # write some throttling values to the config file
        with open(conf_file, 'w') as fp:
            fp.write('[__main__]\n')
            fp.write('log_level = INFO\n')
            fp.write('disable_ssl_verify = True\n')
            fp.write('\n')
            fp.write('[bandwidth_throttling]\n')
            fp.write('on = True\n')
            fp.write('read_limit = 1000\n')
            fp.write('write_limit = 200\n')
        conf = config._Config(conf_file)
        self.assertEquals(True, conf.get_throttling())
        self.assertEquals(1000, conf.get_throttling_read_limit())
        self.assertEquals(200, conf.get_throttling_write_limit())

    def test_write_new(self):
        """test writing the throttling section to a new config file"""
        conf_file = os.path.join(self.test_root, 'test_write_new_config.conf')
        self.assertFalse(os.path.exists(conf_file))
        conf = config._Config(conf_file)
        conf.set_throttling(True)
        conf.set_throttling_read_limit(1000)
        conf.set_throttling_write_limit(100)
        conf.save()
        # load the config in a barebone ConfigParser and check
        conf_1 = ConfigParser()
        conf_1.read(conf_file)
        self.assertThrottlingSection(conf_1, conf, True, 1000, 100)

    def test_write_existing(self):
        """test writing the throttling section to a existing config file"""
        conf_file = os.path.join(self.test_root,
                                 'test_write_existing_config.conf')
        # write some throttling values to the config file
        with open(conf_file, 'w') as fp:
            fp.write('[bandwidth_throttling]\n')
            fp.write('on = False\n')
            fp.write('read_limit = 1000\n')
            fp.write('write_limit = 100\n')
        self.assertTrue(os.path.exists(conf_file))
        conf = config._Config(conf_file)
        conf.set_throttling(True)
        conf.set_throttling_read_limit(2000)
        conf.set_throttling_write_limit(200)
        conf.save()
        # load the config in a barebone ConfigParser and check
        conf_1 = ConfigParser()
        conf_1.read(conf_file)
        self.assertThrottlingSection(conf_1, conf, True, 2000, 200)

    def test_write_extra(self):
        """test writing the throttling section back to the config file,
        including extra sections
        """
        conf_file = os.path.join(self.test_root, 'test_write_extra_config.conf')
        # write some throttling values to the config file
        with open(conf_file, 'w') as fp:
            fp.write('[__main__]\n')
            fp.write('log_level = INFO\n')
            fp.write('disable_ssl_verify = True\n')
            fp.write('\n')
            fp.write('[bandwidth_throttling]\n')
            fp.write('on = False\n')
            fp.write('read_limit = 2000\n')
            fp.write('write_limit = 200\n')
        self.assertTrue(os.path.exists(conf_file))
        conf = config._Config(conf_file)
        conf.set_throttling(True)
        conf.set_throttling_read_limit(3000)
        conf.set_throttling_write_limit(300)
        conf.save()
        # load the config in a barebone ConfigParser and check
        conf_1 = ConfigParser()
        conf_1.read(conf_file)
        self.assertThrottlingSection(conf_1, conf, True, 3000, 300)
        self.assertEquals(conf_1.get('__main__', 'log_level'),
                          conf.get('__main__', 'log_level'))
        self.assertEquals(conf_1.getboolean('__main__', 'disable_ssl_verify'),
                          conf.getboolean('__main__', 'disable_ssl_verify'))

    def test_write_existing_partial(self):
        """test writing a partially updated throttling section
        to a existing config file
        """
        conf_file = os.path.join(self.test_root,
                                 'test_write_existing_config.conf')
        # write some throttling values to the config file
        with open(conf_file, 'w') as fp:
            fp.write('[bandwidth_throttling]\n')
            fp.write('on = True\n')
            fp.write('read_limit = 1000\n')
            fp.write('write_limit = 100\n')
        self.assertTrue(os.path.exists(conf_file))
        conf = config._Config(conf_file)
        conf.set_throttling(False)
        conf.save()
        # load the config in a barebone ConfigParser and check
        conf_1 = ConfigParser()
        conf_1.read(conf_file)
        self.assertThrottlingSection(conf_1, conf, False, 1000, 100)


    def test_load_negative_limits(self):
        """test loading the config file with negative read/write limits"""
        conf_file = os.path.join(self.test_root, 'test_load_config.conf')
        # write some throttling values to the config file
        with open(conf_file, 'w') as fp:
            fp.write('[bandwidth_throttling]\n')
            fp.write('on = True\n')
            fp.write('read_limit = -1\n')
            fp.write('write_limit = -1\n')
        conf = config._Config(conf_file)
        self.assertEquals(True, conf.get_throttling())
        self.assertEquals(None, conf.get_throttling_read_limit())
        self.assertEquals(None, conf.get_throttling_write_limit())


    def test_load_partial_config(self):
        """test loading a partial config file and fallback to defaults"""
        conf_file = os.path.join(self.test_root, 'test_load_config.conf')
        # write some throttling values to the config file
        with open(conf_file, 'w') as fp:
            fp.write('[bandwidth_throttling]\n')
            fp.write('on = True\n')
            fp.write('read_limit = 1\n')
        conf = config._Config(conf_file)
        self.assertEquals(True, conf.get_throttling())
        self.assertEquals(1, conf.get_throttling_read_limit())
        self.assertEquals(None, conf.get_throttling_write_limit())


class ConfigglueParsersTests(BaseTwistedTestCase):
    """Tests for our custom configglue parsers"""

    def test_throttling_limit_parser(self):
        """Test throttling_limit_parser"""
        good_value = '20480'
        unset_value = '-1'
        bad_value = 'hola'
        invalid_value = None
        parser = config.throttling_limit_parser
        self.assertEquals(20480, parser(good_value))
        self.assertEquals(None, parser(unset_value))
        self.assertRaises(ValueError, parser, bad_value)
        self.assertRaises(TypeError, parser, invalid_value)

    def test_log_level_parser(self):
        """Test log_level_parser"""
        good_value = 'INFO'
        bad_value = 'hola'
        invalid_value = None
        parser = config.log_level_parser
        self.assertEquals(logging.INFO, parser(good_value))
        self.assertEquals(logging.DEBUG, parser(bad_value))
        self.assertRaises(TypeError, parser, invalid_value)

    def test_home_dir_parser(self):
        """Test home_dir_parser"""
        good_value = '~/hola'
        bad_value = 'hola'
        invalid_value = None
        parser = config.home_dir_parser
        with environ('HOME', '/home/fake'):
            self.assertEquals('/home/fake/hola', parser(good_value))
        self.assertEquals('hola', parser(bad_value))
        self.assertRaises(AttributeError, parser, invalid_value)

    def test_xdg_cache_dir_parser(self):
        """Test xdg_cache_dir_parser"""
        good_value = 'hola'
        bad_value = '/hola'
        invalid_value = None
        parser = config.xdg_cache_dir_parser
        self.assertEquals(os.path.join(xdg_cache_home, 'hola'),
                          parser(good_value))
        self.assertEquals('/hola', parser(bad_value))
        self.assertRaises(AttributeError, parser, invalid_value)

    def test_xdg_data_dir_parser(self):
        """Test xdg_data_dir_parser"""
        good_value = 'hola'
        bad_value = '/hola'
        invalid_value = None
        parser = config.xdg_data_dir_parser
        self.assertEquals(os.path.join(xdg_data_home, 'hola'),
                          parser(good_value))
        self.assertEquals('/hola', parser(bad_value))
        self.assertRaises(AttributeError, parser, invalid_value)


class TestDBusConfig(DBusTwistedTestCase):

    def setUp(self):
        """Fake the user_config file, and setup all SD stuff"""
        self.old_get_config_files = config.get_config_files
        config.get_config_files = lambda: [os.path.join(os.environ['ROOT_DIR'],
                                                   'data', 'syncdaemon.conf')]
        return DBusTwistedTestCase.setUp(self)

    def tearDown(self):
        """cleanup everything, also the config module globals"""
        config._user_config = None
        config.get_config_files = self.old_get_config_files
        return DBusTwistedTestCase.tearDown(self)

    def test_get_throttling_limits(self):
        """Test getting throttling limits"""
        dbus_config = self.main.dbus_iface.config
        limits = dbus_config.get_throttling_limits()
        self.assertEquals(limits['download'], -1)
        self.assertEquals(limits['upload'], -1)

    def test_set_throttling_limits(self):
        """Test setting throttling limits"""
        dbus_config = self.main.dbus_iface.config
        dbus_config.set_throttling_limits(10, 20)
        limits = dbus_config.get_throttling_limits()
        self.assertEquals(limits['download'], 10)
        self.assertEquals(limits['upload'], 20)
        conf = ConfigParser()
        conf.read(config.get_user_config().config_file)
        self.assertEquals(conf.get(config.THROTTLING, 'read_limit'), '10')
        self.assertEquals(conf.get(config.THROTTLING, 'write_limit'), '20')

    def test_enable_bandwidth_throttling(self):
        """Test for enabling bandwidth throttling"""
        dbus_config = self.main.dbus_iface.config
        enabled = dbus_config.bandwidth_throttling_enabled()
        self.assertFalse(enabled)
        dbus_config.enable_bandwidth_throttling()
        enabled = dbus_config.bandwidth_throttling_enabled()
        self.assertTrue(enabled)
        conf = ConfigParser()
        conf.read(config.get_user_config().config_file)
        self.assertEquals(conf.get(config.THROTTLING, 'on'), 'True')

    def test_disable_bandwidth_throttling(self):
        """Test for disabling bandwidth throttling"""
        dbus_config = self.main.dbus_iface.config
        enabled = dbus_config.bandwidth_throttling_enabled()
        self.assertFalse(enabled)
        # enable throttling
        dbus_config.enable_bandwidth_throttling()
        enabled = dbus_config.bandwidth_throttling_enabled()
        self.assertTrue(enabled)
        # now disable throttling and check
        dbus_config.disable_bandwidth_throttling()
        enabled = dbus_config.bandwidth_throttling_enabled()
        self.assertFalse(enabled)
        conf = ConfigParser()
        conf.read(config.get_user_config().config_file)
        self.assertEquals(conf.get(config.THROTTLING, 'on'), 'False')
